
/* ------------------------------ fileio.c --------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include "main.h"
#include "fileio.h"

/* Print file I/O error message and exit. "file" is 0 for the input,	*/
/* and 1 for the output file.						*/

void	err_msg_file (char *msg, int file)
{
	char	*filename, *err_str;

	switch (file) {
		case 0:	if (in_file_name == NULL) {
				filename = "stdin";
			} else {
				filename = in_file_name;
			}
			break;
		case 1:	if (out_file_name == NULL) {
				filename = "stdout";
			} else {
				filename = out_file_name;
			}
			break;
		default:	filename = NULL;
	}
	err_str = strerror (errno);
	if (filename == NULL) {
		fprintf (stderr, "snd_resample: %s\n", err_str);
	} else {
		fprintf (stderr, "snd_resample: %s %s: %s\n",
				 msg, filename, err_str);
	}
	exit (-1);
}

/* Open input and output file with error checking.			*/

void	open_files (void)
{
	if (in_file_name == NULL) {
		in_file = stdin;
	} else {
		in_file = fopen (in_file_name, "rb");
		if (in_file == NULL) err_msg_file ("error opening", 0);
	}

	if (out_file_name == NULL) {
		out_file = stdout;
	} else {
		out_file = fopen (out_file_name, "wb");
		if (out_file == NULL) err_msg_file ("error opening", 1);
	}
}

/* Close input and output file.						*/

void	close_files (void)
{
	if (in_file_name != NULL)
		if (fclose (in_file))
			err_msg_file ("error closing", 0);

	fflush (out_file);
	if (out_file_name != NULL)
		if (fclose (out_file))
			err_msg_file ("error closing", 1);
}

/* Read nbytes bytes from input file to buf with checking for EOF and	*/
/* error. Return value is the number of bytes read (may be less than	*/
/* nbytes at end of file).						*/

long	file_read (void *buf, long nbytes)
{
	long	nrem;

	nrem = nbytes;
	while ((nrem > 0) && (!(infile_eof))) {
		nrem -= fread (buf, 1, nrem, in_file);
		if (ferror (in_file))		/* check for error */
			err_msg_file ("error reading", 0);
		if (feof (in_file))		/* check for end of file */
			infile_eof = 1;
	}

	return (nbytes - nrem);
}

/* Write nbytes bytes from buf to output file with checking for error.	*/

void	file_write (void *buf, long nbytes)
{
	long	nrem;

	nrem = nbytes;
	while (nrem > 0) {
		nrem -= fwrite (buf, 1, nrem, out_file);
		if (ferror (out_file))		/* check for error */
			err_msg_file ("error writing to", 1);
	}
}

/* Read next BUFFER_SIZE / 4 samples from input file.			*/

void	read_block (void)
{
	int		i, j, k, fsamps, bs_d2, bs_d4, bs_m3d4;
	signed short	*io_buf_s;
	float		*io_buf_f;

	io_buf_s = (signed short *) io_buf;
	io_buf_f = (float *) io_buf;

	bs_d2 = BUFFER_SIZE >> 1;		/* buffer size / 2	*/
	bs_d4 = BUFFER_SIZE >> 2;		/* buffer size / 4	*/
	bs_m3d4 = (BUFFER_SIZE * 3) >> 2;	/* buffer size * 3/4	*/

	/* move data in buffer (BUFFER_SIZE / 4 samples back) */

	i = 0; j = bs_d4 * nchnls; k = bs_m3d4 * nchnls;
	while (i < k) in_buf[i++] = in_buf[j++];
	while (i < j) in_buf[i++] = 0.0;

	/* end of file: nothing to read */

	if ((infile_eof) && (io_buf_pos == 0)) return;

	i = k; while ((i < j) && (!(infile_eof))) {

		/* clear I/O buffer */

		for (k = io_buf_pos; k < BLK_SIZE; k++) io_buf[k] = (char) 0;

		/* read data from file */

		k = file_read ((void *) (io_buf + io_buf_pos),
				(long) BLK_SIZE - (long) io_buf_pos);
		k += io_buf_pos;
		io_buf_pos = 0;

		/* fsamps = number of samples read * nchnls */

		fsamps = (k + in_samp_size - 1) / in_samp_size;
		infile_size += (fsamps + nchnls - 1) / nchnls;

		if (infile_eof)			/* end of file */
			outfile_size = (int)
				((double) infile_size * resample_ratio + 0.5);

		/* copy data to input buffer */

		if ((fsamps += i) > j) {
			fprintf (stderr, "snd_resample: ");
			fprintf (stderr, "fsamps > BUFFER_SIZE\n"); exit (-1);
		}

		k = 0;
		if (in_samp_size == sizeof (short int)) {	/* shorts */
			while (i < fsamps) in_buf[i++] = (double) io_buf_s[k++];
		} else {					/* floats */
			while (i < fsamps) in_buf[i++] = (double) io_buf_f[k++];
		}
	}
}

/* convert the double value x to signed short int (also add dither noise) */

signed short int
	double2short (double x)
{
	long int		n;
	double			y;
	signed short int	z;

	n = rand ();
	n -= (RAND_MAX >> 1);
	y = (double) n / (double) (RAND_MAX);
	y += x;
	y += (y > 0.0 ? 0.5 : -0.5);
	y = (y > 32767.0 ? 32767.0 : y);
	y = (y < -32767.0 ? -32767.0 : y);
	z = (signed short int) y;

	return z;
}

/* Write data to output file.						*/

void	write_block (void)
{
	int		i, j, k, nsamps, fsamps, s, bs_d2, bs_d4, bs_m3d4;
	signed short	*io_buf_s;
	float		*io_buf_f;

	io_buf_s = (signed short *) io_buf;
	io_buf_f = (float *) io_buf;

	bs_d2 = BUFFER_SIZE >> 1;		/* buffer size / 2	*/
	bs_d4 = BUFFER_SIZE >> 2;		/* buffer size / 4	*/
	bs_m3d4 = (BUFFER_SIZE * 3) >> 2;	/* buffer size * 3/4	*/

	/* fsamps = total number of samples to write * nchnls */

	fsamps = outfile_size - outfile_pos;
	fsamps = (fsamps > bs_d4 ? bs_d4 : fsamps) * nchnls;

	/* nsamps = number of samples to write at once * nchnls */

	nsamps = (int) BLK_SIZE / out_samp_size;

	i = bs_d2 * nchnls; j = i + fsamps;
	while (i < j) {

		/* s = number of samples to write * nchnls */

		s = j - i; if (s > nsamps) s = nsamps;

		/* copy data to I/O buffer, and write to output file */

		k = 0;
		if (out_samp_size == (sizeof (short int))) {
			while (k < s)		/* short int samples	*/
				io_buf_s[k++] = double2short (out_buf[i++]);
		} else {
			while (k < s)		/* float samples	*/
				io_buf_f[k++] = (float) out_buf[i++];
		}
		file_write ((void *) io_buf, s * out_samp_size);
	}
	outfile_pos += fsamps / nchnls;

	/* move data in buffer (BUFFER_SIZE / 4 samples back) */

	i = 0; j = bs_d4 * nchnls; k = bs_m3d4 * nchnls;
	while (i < k) out_buf[i++] = out_buf[j++];
	while (i < j) out_buf[i++] = 0.0;
}

/* Check if input file is IRCAM format, and read parameters from header. */

void	read_header (void)
{
	SFHEADER	*sf_hdr;

	io_buf_pos = file_read ((void *) io_buf, SFHEADER_SIZE);
	sf_hdr = (SFHEADER *) io_buf;
	if (sf_hdr->magic != 0x0001A364L) {
		if (sf_hdr->magic == 0x64A30100L) {
			fprintf (stderr, "snd_resample: IRCAM file with ");
			fprintf (stderr, "wrong byteorder\n");
			exit (-1);
		}
		/* assume raw data */
		infile_format = 0;
	} else {
		/* IRCAM format */
		infile_format = 1;
		if (io_buf_pos < SFHEADER_SIZE) {
			fprintf (stderr, "snd_resample: ");
			fprintf (stderr, "truncated IRCAM file\n");
			exit (-1);
		}
		/* skip header */
		io_buf_pos = 0;
		/* set sample rate, format, and nchnls according to header */
		infile_sr = (float) sf_hdr->sr;
		in_samp_size = (int) sf_hdr->format;
		if (in_samp_size == sizeof (signed short)) {
			in_format = 0;
		} else if (in_samp_size == sizeof (float)) {
			in_format = 1;
		} else {
			fprintf (stderr, "snd_resample: unsupported sample ");
			fprintf (stderr, "format\n");
			exit (-1);
		}
		nchnls = (int) sf_hdr->nchnls;
	}
}

/* Write output file header if output format is IRCAM.			*/

void	write_header (void)
{
	SFHEADER	*sf_hdr;
	short	*x;

	if (outfile_format != 1) return;	/* raw output file */
	/* allocate memory for header */
	sf_hdr = (SFHEADER *) calloc (1, sizeof (SFHEADER));
	if (sf_hdr == NULL) {
		fprintf (stderr, "snd_resample: not enough memory\n");
		exit (-1);
	}
	/* set parameters */
	sf_hdr->magic = 0x0001A364L;
	sf_hdr->sr = (float) outfile_sr;
	sf_hdr->format = (long) out_samp_size;
	sf_hdr->nchnls = (long) nchnls;

	x = (short *) &(sf_hdr->sfcode);
	/* sample format */
	*(x++) = 0x0002;
	*(x++) = 0x0008;
	if (out_samp_size == sizeof (short)) {
		*(x++) = 0x0104;		/* short int	*/
	} else {
		*(x++) = 0x0106;		/* float	*/
	}
	*(x++) = 0x0001;
	/* end of header */
	*(x++) = 0x0000;
	*(x++) = 0x0004;

	/* write to file */
	file_write (sf_hdr, SFHEADER_SIZE);

	free (sf_hdr); sf_hdr = NULL;
}

