
/* ------------------------------- main.c ---------------------------------- */

/* snd_resample v1.1			*/
/* Written by Istvan Varga, Oct 2001	*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include "main.h"
#include "interp.h"
#include "fileio.h"
#include "opts.h"

int	interp_size	=  64;		/* interpolation window size	     */
double	resample_ratio	=  -1.0;	/* resample ratio		     */
int	in_format	=  1;		/* input sample format (1: float)    */
int	out_format	=  1;		/* output sample format (1: float)   */
int	nchnls		=  1;		/* number of channels		     */
int	v_mode		=  0;		/* verbose mode			     */

int	in_samp_size	=  4;		/* input sample size		     */
int	out_samp_size	=  4;		/* output sample size		     */

FILE	*in_file	=  NULL;	/* input file			     */
FILE	*out_file	=  NULL;	/* output file			     */
char	*in_file_name	=  NULL;	/* input file name		     */
char	*out_file_name	=  NULL;	/* output file name		     */
int	infile_format	=  0;		/* input file format		     */
int	outfile_format	=  0;		/* output file format		     */
float	infile_sr	=  44100.0f;	/* input file sample rate	     */
float	outfile_sr	=  44100.0f;	/* output file sample rate	     */

double	*in_buf;			/* input buffer			     */
double	*out_buf;			/* output buffer		     */
char	*io_buf;			/* low level I/O buffer		     */
int	io_buf_pos;			/* nr of unread bytes in io_buf	     */

int	infile_size;			/* input file size in samples	     */
int	outfile_size;			/* output file size in samples	     */
int	infile_eof;			/* end of input file (0: no, 1: yes) */
int	infile_pos;			/* input file position (samples)     */
int	outfile_pos;			/* output file position (samples)    */
int	in_buf_pos;			/* input buffer position	     */
int	out_buf_pos;			/* output buffer position	     */

int	downsample_mode	=  0;		/* downsample mode (0: off, 1: on)   */
int	resample_int	=  0;		/* integer resample ratio (1: yes)   */
double	win_fact;			/* constant for window calculation   */

int	main (int argc, char **argv)
{
	double	p;
	int	bs_d2, bs_d4, bs_m3d4, pos0;

	io_buf = (char *) malloc (BLK_SIZE);	/* I/O buffer */
	if (io_buf == NULL) {
		fprintf (stderr, "snd_resample: not enough memory\n");
		exit (-1);
	}

	srand (16);			/* set random seed		     */
	read_opts (argc, argv);		/* parse command line options	     */
	open_files ();			/* open input and output file	     */
	read_header ();			/* check input file header	     */

	/* check parameters */

	if ((nchnls != 1) && (nchnls != 2) && (nchnls != 4)) {
		fprintf (stderr, "snd_resample: invalid number of channels\n");
		exit (-1);
	}
	if (infile_sr <= 0.0f) {
		fprintf (stderr, "snd_resample: invalid sample rate: %.2f\n",
				 infile_sr);
		exit (-1);
	}
	if (resample_ratio <= 0.0) {
		resample_ratio = (double) outfile_sr / (double) infile_sr;
		if ((resample_ratio < 0.0625) || (resample_ratio > 16.0)) {
			fprintf (stderr, "snd_resample: ");
			fprintf (stderr, "invalid resample ratio: %f\n",
					 resample_ratio);
			exit (-1);
		}
	} else {
		outfile_sr = (float) (resample_ratio * (double) infile_sr);
	}
	in_samp_size = (in_format ? sizeof (float) : sizeof (short int));
	out_samp_size = (out_format ? sizeof (float) : sizeof (short int));
	downsample_mode = (resample_ratio < 1.0 ? 1 : 0);
	p = (downsample_mode ? 1.0 / resample_ratio : resample_ratio);
	p = fabs ((p / (double) ((long) (p + 0.5))) - 1.0);
	resample_int = (p < 5.0e-7 ? 1 : 0);
	win_fact = 4.0 / (double) (interp_size * interp_size);
	win_fact *= 1.0 - pow ((double) interp_size * 0.85172, -0.89624);

	print_settings ();		/* print settings in verbose mode    */
	write_header ();		/* write output file header	     */
	initialize_window_tables ();	/* set up pre-calculated tables	     */

	/* allocate space for buffers */

	in_buf = (double *) calloc (nchnls * BUFFER_SIZE, sizeof (double));
	if (in_buf == NULL) {
		fprintf (stderr, "snd_resample: not enough memory\n");
		exit (-1);
	}
	out_buf = (double *) calloc (nchnls * BUFFER_SIZE, sizeof (double));
	if (out_buf == NULL) {
		fprintf (stderr, "snd_resample: not enough memory\n");
		exit (-1);
	}

	bs_d2 = BUFFER_SIZE >> 1;
	bs_d4 = BUFFER_SIZE >> 2;
	bs_m3d4 = (BUFFER_SIZE >> 1) + (BUFFER_SIZE >> 2);

	infile_size = 0;
	outfile_size = 0x7FFFFFFF;
	infile_eof = 0;
	infile_pos = 0;
	outfile_pos = 0;
	in_buf_pos = BUFFER_SIZE;
	out_buf_pos = bs_d2;
	p = (downsample_mode ? (double) bs_d2 : (double) BUFFER_SIZE);
	pos0 = bs_m3d4 + (downsample_mode ? interp_size : 0);

	do {	/* read data from input file */

		while (in_buf_pos >= bs_m3d4) {
			read_block ();
			in_buf_pos -= bs_d4;
			if (!(downsample_mode)) {
				p -= (double) bs_d4;
			}
		}

		if (downsample_mode) {		/* downsample	*/

			if (resample_int) {
				downsample_int (in_buf_pos, p);
			} else {
				downsample (in_buf_pos, p);
			}

			in_buf_pos++;
			p += resample_ratio;
			out_buf_pos = (int) p;

		} else {			/* upsample	*/

			if (resample_int) {
				upsample_int (p, out_buf_pos);
			} else {
				upsample (p, out_buf_pos);
			}

			out_buf_pos++;
			p += (1.0 / resample_ratio);
			in_buf_pos = (int) p;
		}

		/* write to output file */

		while (out_buf_pos >= pos0) {
			write_block ();
			out_buf_pos -= bs_d4;
			if (downsample_mode) p -= (double) bs_d4;
			if (v_mode)
				fprintf (stderr, "\r%d samples written    ",
						outfile_pos);
		}

	} while (outfile_pos < outfile_size);

	if (v_mode) fprintf (stderr, "\n");

	close_files ();			/* close all files		*/
	free_window_tables ();		/* free memory used by tables	*/
	free (out_buf); out_buf = NULL;
	free (in_buf); in_buf = NULL;
	free (io_buf); io_buf = NULL;

	return 0;
}

