
/* -------------------------------- opts.c --------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include "main.h"
#include "opts.h"

/* list of all available options */

char	*opt_list[] = { "-i", "-o", "-r", "-s",			/*  1 -  4 */
			 "-f", "-c1", "-c2", "-c4",		/*  5 -  8 */
			 "-R", "-S", "-F", "-J",		/*  9 - 12 */
			 "-q12", "-q16", "-q24", "-q32",	/* 13 - 16 */
			 "-q48", "-q64", "-q96", "-q128",	/* 17 - 20 */
			 "-q192", "-q256", "-q384", "-q512",	/* 21 - 24 */
			 "-q768", "-q1024", "-v", "-h",		/* 25 - 28 */
			 NULL };

/* print help and exit */

void	print_usage (void)
{
	fprintf (stderr, "snd_resample v1.1 - written by Istvan Varga, ");
	fprintf (stderr, "Oct 2001\n\n");
	fprintf (stderr, "usage: snd_resample [options...]\n");
	fprintf (stderr, "\n");
	fprintf (stderr, "file name options:\n");
	fprintf (stderr, "    -i <input file> (default: read from stdin)\n");
	fprintf (stderr, "    -o <output file> (default: write to stdout)\n");
	fprintf (stderr, "raw input file options:\n");
	fprintf (stderr, "    -r:   sample rate (default: 44100 Hz)\n");
	fprintf (stderr, "    -s:   short int samples\n");
	fprintf (stderr, "    -f:   float samples (default)\n");
	fprintf (stderr, "    -c1:  mono input\n");
	fprintf (stderr, "    -c2:  stereo input\n");
	fprintf (stderr, "    -c4:  quad input\n");
	fprintf (stderr, "output file options:\n");
	fprintf (stderr, "    -R:   sample rate (default: 44100 Hz)\n");
	fprintf (stderr, "    -S:   short int samples (dithered)\n");
	fprintf (stderr, "    -F:   float samples (default)\n");
	fprintf (stderr, "    -J:   IRCAM format output (default: raw)\n");
	fprintf (stderr, "resample options:\n");
	fprintf (stderr, "    -qnn: use \"nn\" samples for interpolation\n");
	fprintf (stderr, "          allowed values for \"nn\" are ");
	fprintf (stderr, "12, 16, 24, 32, 48, 64,\n");
	fprintf (stderr, "          96, 128, 192, 256, 384, 512, 768, 1024\n");
	fprintf (stderr, "    nn:   set resample ratio to \"nn\" ");
	fprintf (stderr, "(overrides -R)\n");
	fprintf (stderr, "          the allowed range is 1/16 to 16\n");
	fprintf (stderr, "misc. options:\n");
	fprintf (stderr, "    -v:   verbose mode\n");
	fprintf (stderr, "    -h:   print this message\n");
	exit (0);
}

/* print error message for missing command line argument */

void	err_msg_noarg (char *argname)
{
	fprintf (stderr, "snd_resample: missing argument for %s\n", argname);
	exit (-1);
}

/* parse command line options */

void	read_opts (int argc, char **argv)
{
	int	i, j;

	i = 0; while (++i < argc) {
		j = 0; while (opt_list[j] != NULL) {
			if (!(strcmp (opt_list[j], argv[i]))) break;
			j++;
		}
		if (opt_list[j] == NULL) j = -1;
		j++;

		switch (j) {
			case  1:			/* input file name   */
				if (++i >= argc) err_msg_noarg ("-i");
				in_file_name = argv[i]; break;
			case  2:			/* output file name  */
				if (++i >= argc) err_msg_noarg ("-o");
				out_file_name = argv[i]; break;
			case  3:			/* input sr	     */
				if (++i >= argc) err_msg_noarg ("-r");
				infile_sr = (float) atof (argv[i]); break;
			case  4:			/* short int input   */
				in_format = 0; break;
			case  5:			/* float input	     */
				in_format = 1; break;
			case  6:			/* mono input	     */
				nchnls = 1; break;
			case  7:			/* stereo input	     */
				nchnls = 2; break;
			case  8:			/* quad input	     */
				nchnls = 4; break;
			case  9:			/* output sr	     */
				if (++i >= argc) err_msg_noarg ("-R");
				outfile_sr = (float) atof (argv[i]); break;
			case 10:			/* short int output  */
				out_format = 0; break;
			case 11:			/* float output	     */
				out_format = 1; break;
			case 12:			/* IRCAM output	     */
				outfile_format = 1; break;
						/* interpolation quality     */
			case 13:	interp_size =   12; break;
			case 14:	interp_size =   16; break;
			case 15:	interp_size =   24; break;
			case 16:	interp_size =   32; break;
			case 17:	interp_size =   48; break;
			case 18:	interp_size =   64; break;
			case 19:	interp_size =   96; break;
			case 20:	interp_size =  128; break;
			case 21:	interp_size =  192; break;
			case 22:	interp_size =  256; break;
			case 23:	interp_size =  384; break;
			case 24:	interp_size =  512; break;
			case 25:	interp_size =  768; break;
			case 26:	interp_size = 1024; break;
			case 27:			/* verbose mode	     */
				v_mode = 1; break;
			case 28:			/* print help	     */
				print_usage (); break;
			default:			/* resample ratio    */
				resample_ratio = (double) atof (argv[i]);
				if ((resample_ratio < 0.0625)
				    || (resample_ratio > 16.0)) {
					fprintf (stderr, "snd_resample: ");
					fprintf (stderr, "invalid option: ");
					fprintf (stderr, "%s\n", argv[i]);
					exit (-1);
				}
		}
	}
}

/* print settings in verbose mode */

void	print_settings (void)
{
	if (!(v_mode)) return;

	fprintf (stderr, "snd_resample v1.1 - written by Istvan Varga, ");
	fprintf (stderr, "Oct 2001\n\n");

	if (in_file_name == NULL) {
		fprintf (stderr, "input file:            standard input\n");
	} else {
		fprintf (stderr, "input file:            %s\n", in_file_name);
	}
	fprintf (stderr, "input sample format:   ");
	if (in_format) {
		fprintf (stderr, "float\n");
	} else {
		fprintf (stderr, "short int\n");
	}
	if (out_file_name == NULL) {
		fprintf (stderr, "output file:           standard output\n");
	} else {
		fprintf (stderr, "output file:           %s\n", out_file_name);
	}
	fprintf (stderr, "output file format:    ");
	if (outfile_format) {
		fprintf (stderr, "IRCAM\n");
	} else {
		fprintf (stderr, "raw\n");
	}
	fprintf (stderr, "output sample format:  ");
	if (out_format) {
		fprintf (stderr, "float\n");
	} else {
		fprintf (stderr, "short int (dithered)\n");
	}
	fprintf (stderr, "number of channels:    %d\n", nchnls);
	fprintf (stderr, "input sample rate:     %.2f\n", (double) infile_sr);
	fprintf (stderr, "output sample rate:    %.2f\n", (double) outfile_sr);
	fprintf (stderr, "resample ratio:        %f\n", resample_ratio);
	fprintf (stderr, "interpolation quality: %d\n", interp_size);
}

